<?PHP
/**
 * Represents a type a transcription was generated towards.
 *
 * @author Joshua Ramon Enslin <joshua@museum-digital.de>
 */
declare(strict_types = 1);

/**
 * Represents a type a transcription was generated towards.
 */
enum MDTranscriptionType implements MDValueEnumInterface, JsonSerializable {

    case simple_transcription;
    case diplomatic_transcription;
    case critical_edition;
    case historical_critical_edition;
    case reading_edition;

    /**
     * Returns a value of this type based on a string.
     *
     * @param string $input Input to get a value from.
     *
     * @return MDTranscriptionType
     */
    public static function fromString(string $input):MDTranscriptionType {

        return match($input) {
            'simple_transcription' => self::simple_transcription,
            'diplomatic_transcription' => self::diplomatic_transcription,
            'critical_edition' => self::critical_edition,
            'historical_critical_edition' => self::historical_critical_edition,
            'reading_edition' => self::reading_edition,
            default => throw new MDpageParameterNotFromListException("Unknown transcription type"),
        };

    }

    /**
     * Returns a value of this type based on an integer.
     *
     * @param integer $input Input to get a value from.
     *
     * @return MDTranscriptionType
     */
    public static function fromInt(int $input):MDTranscriptionType {

        return match($input) {
            1 => self::simple_transcription,
            2 => self::diplomatic_transcription,
            3 => self::critical_edition,
            4 => self::historical_critical_edition,
            5 => self::reading_edition,
            default => throw new MDpageParameterNotFromListException("Unknown transcript type"),
        };

    }

    /**
     * Lists all available names.
     *
     * @return array<string>
     */
    public static function caseNames():array {

        $output = [];

        $cases = self::cases();
        foreach ($cases as $case) {
            $output[] = $case->name;
        }

        return $output;

    }

    /**
     * Returns an integer representation of the collective (for storage in DB).
     *
     * @return integer
     */
    public function toInt():int {

        return match($this) {
            self::simple_transcription => 1,
            self::diplomatic_transcription => 2,
            self::critical_edition => 3,
            self::historical_critical_edition => 4,
            self::reading_edition => 5,
            # default => throw new MDpageParameterNotFromListException("Unknown TranscriptionTypee"),
        };

    }

    /**
     * Gets an unsorted list of the entries in a translated version.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return array<string, string>
     */
    public static function getUnsortedList(MDTlLoader $tlLoader):array {
        return MDValueSet::getTlUnsortedList($tlLoader, self::caseNames(), "transcript_types", "transcript_types");

    }

    /**
     * Gets a sorted list of the entries in a translated version.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return array<string, string>
     */
    public static function getSortedList(MDTlLoader $tlLoader):array {
        return MDValueSet::getTlSortedList($tlLoader, self::caseNames(), "transcript_types", "transcript_types");

    }

    /**
     * Returns the name of the current value in translation.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return string
     */
    public function getTledName(MDTlLoader $tlLoader):string {

        return $tlLoader->tl("transcript_types", "transcript_types", $this->name);

    }

    /**
     * Provides the option to serialize as a string during json_encode().
     *
     * @return string
     */
    public function jsonSerialize():string {

        return $this->name;

    }
}
